node-weak
=========
### Make weak references to JavaScript Objects with an associated Callback function before GC.

On certain rarer occasions, you run into the need to be notified when a JavaScript
object is going to be garbage collected. This feature is exposed to V8's C++ API,
but not to JavaScript.

That's where `node-weak` comes in! This module exports a single function,
`weak()`, which allows you to attach a callback Function to any arbitrary JS
object. The callback function will be invoked right before the Object is destroyed
and garbage collected (i.e. after there are no more remaining references to the
object in JS-land).

Take a look at the example below for a simple use-case.


Installation
------------

Install with `npm`:

``` shell
$ npm install weak
```


Example
-------

Here's an example of calling a `cleanup()` function on a Object before it gets
garbage collected:

``` js
var weak = require('weak')

// we are going to "monitor" this Object an invoke "cleanup" before
// the object is garbage collected
var obj = {
  , a: true
  , foo: 'bar'
  , cleanup: function () {
      delete this.a
      delete this.foo
    }
}

// Here's where we set up the weak reference
weak(obj, function () {
  // `this` inside the callback is the 'obj'. DO NOT store any new references
  // to the object, and DO NOT use the object in any async functions.
  this.cleanup()
})


// Clear out any references to the object, so that it will be GC'd at some point...
obj = null
```
