#include <v8.h>
#include <node.h>

using namespace v8;

// Callback that gets invoked right before the object is getting garbage collected
static void WeakCallback(Persistent<Value> object, void *hint) {

  Persistent<Function> cb = *reinterpret_cast<Persistent<Function>*>(hint);

  TryCatch tryCatch;
  cb->Call(object->ToObject(), 0, NULL);
  if (tryCatch.HasCaught()) {
    //FatalException(tryCatch);
  }

  // Clear up the Persistent handles
  cb.Dispose();
  cb.Clear();
  object.Dispose();
  object.Clear();
  free(hint);
}

static Handle<Value> Weak(const Arguments& args) {
  HandleScope scope;

  // Craete a Persistent handle to the Value passed in
  Persistent<Value> val = Persistent<Value>::New(args[0]);

  // Get a Persistent handle to the callback that should be invoked
  Local<Function> cb = Local<Function>::Cast(args[1]);
  Persistent<Function>* pcb = (Persistent<Function>*)malloc(sizeof(Persistent<Function>));
  *pcb = Persistent<Function>::New(cb);

  // Make the persistent handle be a weak reference
  val.MakeWeak(pcb, WeakCallback);

  return Undefined();
}

extern "C" void init (Handle<Object> target) {
  HandleScope scope;
  NODE_SET_METHOD(target, "weak", Weak);
}
